#ifndef __SD_H__
#define __SD_H__

#ifdef __cplusplus
extern "C" {
#endif
#define SD_NO_DETECT_PIN

#define IO_SD_DETECT             ('G')
#define PIN_SD_DETECT            8  // GPG8
#define IO_SD_PROTECT           ('H')
#define PIN_SD_PROTECT          8  // GPH8



//#################################### Register Constants/Masks/etc. ######################################

// NOTE: the maximum SD speed allowed value of 25 MHz.  
#define MAX_SDI_BUS_TRANSFER_SPEED	25000000

#define BYTES_PER_SECTOR				512
#define SDI_CONTROLLER_FIFO_SIZE		64

//----- Register definitions for SDICON control register (global config register) -----
#define LITTLE_ENDIAN_BYTE_ORDER		0x00000000
#define SDMMC_RESET					(DWORD)(1 << 8)
#define SDCLK_TYPE_SD					(DWORD)(0 << 5)
#define SDCLK_TYPE_MMC					(DWORD)(1 << 5)
#define SD_TYPE_A                                    (DWORD)(0 << 4)  // byte order
#define SD_TYPE_B                                    (DWORD)(1 << 4)  // byte order 
#define SDIO_INTERRUPT_ENABLE			(DWORD)(1 << 3)
#define SDIO_READ_WAIT_ENABLE			(DWORD)(1 << 2)
//#define RESET_FIFO						0x00000002 
#define CLOCK_ENABLE					(DWORD)(1 << 0)


//----- Register definitions for SDICMDCON control register (command config register) -----
//		NOTE:  Lowest 8 bits are used for the actual command's opcode
#define SDIO_ABORT_COMMAND			(DWORD)(1 << 12)				// Works for CMD12 and CMD52
#define SDIO_COMMAND_WITH_DATA		(DWORD)(1 << 11)
#define LONG_RESPONSE					(DWORD)(1 << 10)
#define WAIT_FOR_RESPONSE				(DWORD)(1 << 9)
#define START_COMMAND					(DWORD)(1 << 8)

#define COMMAND_START_BIT				0x00000040				// Commands are 6 bits, but the 7th bit must be set!
#define MAX_CMD_VALUE					0x0000007F				// In SD/MMC clock cycles
#define NULL_CMD_ARGUMENT				0x00000000
															
//----- Register definitions for SDICMDSTA control register (command status register) -----
//		NOTE:  Lowest 8 bits indicate the command's response code
#define CRC_CHECK_FAILED				(DWORD)(1 << 12)				
#define COMMAND_SENT					(DWORD)(1 << 11)
#define COMMAND_TIMED_OUT				(DWORD)(1 << 10)
#define RESPONSE_RECEIVED				(DWORD)(1 << 9)
#define COMMAND_IN_PROGRESS			(DWORD)(1 << 8)

#define SHORT_RESPONSE_LENGTH			0x00000006				// In bytes
#define LONG_RESPONSE_LENGTH			0x00000010				// In bytes


//----- Register definitions for SDIDTIMER control register (data/busy config register) -----
//		NOTES:	1) Valid range		= 0-8388607
//				2) Default value	= 0x2000 
#define NO_DATABUSY_TIMEOUT			0
#define MAX_DATABUSY_TIMEOUT			0x007FFFFF

//----- Register definitions for SDIDATCON control register (transfer config register) -----
//		NOTE:  Lowest 12 bits are used for the block number (don't care in stream mode)
#define BURST4_ENABLE					(DWORD)(1 << 24)
#define DATA_SIZE_WORD					(DWORD)(2 << 22)
#define DATA_SIZE_HWORD				(DWORD)(1 << 22)
#define DATA_SIZE_BYTE					(DWORD)(0 << 22)
#define SDIO_INTERRUPT_PERIOD_TYPE		(DWORD)(1 << 21)  // 0 = 2 cycles, 1 = more than 2 cycles
#define TRANSMIT_AFTER_RESPONSE		(DWORD)(1 << 20)  // 
#define RECEIVE_AFTER_COMMAND			(DWORD)(1 << 19)  // 
#define BUSY_AFTER_COMMAND			(DWORD)(1 << 18)
#define TRANSFER_BLOCK_MODE			(DWORD)(1 << 17) // Clearing this bit enables stream mode
#define WIDE_BUS_ENABLE				(DWORD)(1 << 16) // 0=standard bus mode. 1= wide bus mode.
#define DMA_ENABLE						(DWORD)(1 << 15)
//jayson #define STOP_BY_FORCE			0x00004000				
#define DATA_TRANS_START				(DWORD)(1 << 14)

// Bits 12-13 are used to determine transfer mode:
#define DATA_TRANSMIT_START			(DWORD)(3 << 12)
#define DATA_RECEIVE_START				(DWORD)(2 << 12)
#define DATA_BUSY						(DWORD)(1 << 12)
#define DATA_READY						(DWORD)(0 << 12)


//----- Register definitions for SDIDATSTA control register (transfer status register) -----
#define NO_BUSY							(DWORD)(1 << 11)
#define READ_WAIT_REQUEST_OCCURED	(DWORD)(1 << 10)
#define SDIO_INTERRUPT_DETECTED		(DWORD)(1 << 9)
//#define FIFO_FAIL_ERROR					0x00000100
#define DATA_TRANSMIT_CRC_ERROR		(DWORD)(1 << 7)
#define DATA_RECEIVE_CRC_ERROR		(DWORD)(1 << 6)
#define DATA_TIME_OUT					(DWORD)(1 << 5)
#define DATA_TRANSMIT_FINISHED			(DWORD)(1 << 4)
#define BUSY_CHECKS_FINISH				0x00000008
#define BUSY_FINISH                                 (DWORD)(1 << 3)
#define DATA_TRANSMIT_IN_PROGRESS		(DWORD)(1 << 1)
#define DATA_RECIEVE_IN_PROGRESS		(DWORD)(1 << 0)

//----- Register definitions for SDIFSTA control register (FIFO status register) -----
//		NOTE:  Lowest 7 bits indicate the number of data bytes in the FIFO
#define FIFO_RESET						(DWORD)(1 << 16)
#define FIFO_FAIL_ERROR					(DWORD)(3 << 14) //both 01 and 10 = fifo fail.
#define FIFO_AVAIL_FOR_TX				(DWORD)(1 << 13)
#define FIFO_AVAIL_FOR_RX				(DWORD)(1 << 12)
#define TX_FIFO_HALF_FULL				(DWORD)(1 << 11)			// < 33 bytes
#define TX_FIFO_EMPTY					(DWORD)(1 << 10)			// = 0 bytes
#define RX_FIFO_LAST_DATA_READY		(DWORD)(1 << 9)
#define RX_FIFO_FULL					(DWORD)(1 << 8)			// = 64 bytes
#define RX_FIFO_HALF_FULL				(DWORD)(1 << 7)			// > 33 bytes
// FIFO Count [6:0]


//----- Register definitions for SDIIMSK control register (interrupt mask register) -----
#define NOBUSY_INT						    (DWORD)(1 << 18)
#define RESPONSE_CRC_ERROR_INT		    (DWORD)(1 << 17)
#define COMMAND_SENT_INT				    (DWORD)(1 << 16)
#define COMMAND_RESPONSE_TIMEOUT_INT    (DWORD)(1 << 15)
#define COMMAND_RESPONSE_REC_INT		    (DWORD)(1 << 14)
#define READ_WAIT_REQUEST_INT			    (DWORD)(1 << 13)
#define SDIO_HOST_IO_INT				    (DWORD)(1 << 12)
#define FIFO_FAIL_ERROR_INT				    (DWORD)(1 << 11)
#define CRC_STATUS_ERROR_INT			    (DWORD)(1 << 10)
#define DATA_CRC_FAIL_INT				    (DWORD)(1 << 9)
#define DATA_TIMEOUT_INT				    (DWORD)(1 << 8)
#define DATA_COUNTER_ZERO_INT			    (DWORD)(1 << 7)
#define BUSY_CHECKS_COMPLETE_INT		    (DWORD)(1 << 6)
#define START_BIT_ERROR_INT				    (DWORD)(1 << 5)
#define TX_FIFO_HALF_FULL_INT			    (DWORD)(1 << 4)		// < 33 bytes
#define TX_FIFO_EMPTY_INT				    (DWORD)(1 << 3)		// = 0 bytes
#define RX_FIFO_LAST_DATA_READY_INT	    (DWORD)(1 << 2)
#define RX_FIFO_FULL_INT				    (DWORD)(1 << 1)		// = 64 bytes
#define RX_FIFO_HALF_FULL_INT			    (DWORD)(1 << 0)		// > 33 bytes

#define ERROR_INTERRUPTS_HANDLED		COMMAND_RESPONSE_TIMEOUT_INT | DATA_TIMEOUT_INT 


//###################################### Hardware Controller Structures ##################################
typedef enum {
    Idle				= -1,
    CommandSent		= 1,
    CommandComplete	= 2,
    ResponseWait		= 3,
    WriteDataTransfer	= 4,
    WriteDataTransferDone = 5,
    ProgramWait			= 6,
    WriteDataDone		= 7,
    ReadDataTransfer	= 8,
    ReadDataTransferDone = 9,
    ReadDataDone		= 10,

} SDHCDSTATE;

// if not include sdcardddk.h head file
typedef enum  _SD_RESPONSE_TYPE {
    NoResponse  = 0,
    ResponseR1  = 1,
    ResponseR1b = 2,
    ResponseR2  = 3,
    ResponseR3  = 4, 
    ResponseR4  = 5,
    ResponseR5  = 6,
    ResponseR6  = 7,
    ResponseR7  = 8
} SD_RESPONSE_TYPE, *PSD_RESPONSE_TYPE;

/////////////////////////////////////////////////////////////


// Function prototypes


typedef long SD_API_STATUS;
// DDK api status codes
#define SD_API_STATUS_SUCCESS                          ((SD_API_STATUS)0x00000000L)
#define SD_API_STATUS_PENDING                          ((SD_API_STATUS)0x00000001L)
#define SD_API_STATUS_FAST_PATH_SUCCESS                ((SD_API_STATUS)0x00000002L)
#define SD_API_STATUS_FAST_PATH_OPT_SUCCESS            ((SD_API_STATUS)0x00000003L)
#define SD_API_STATUS_BUFFER_OVERFLOW                  ((SD_API_STATUS)0xC0000001L)
#define SD_API_STATUS_DEVICE_BUSY                      ((SD_API_STATUS)0xC0000002L)
#define SD_API_STATUS_UNSUCCESSFUL                     ((SD_API_STATUS)0xC0000003L)
#define SD_API_STATUS_NOT_IMPLEMENTED                  ((SD_API_STATUS)0xC0000004L)
#define SD_API_STATUS_ACCESS_VIOLATION                 ((SD_API_STATUS)0xC0000005L)
#define SD_API_STATUS_INVALID_HANDLE                   ((SD_API_STATUS)0xC0000006L)
#define SD_API_STATUS_INVALID_PARAMETER                ((SD_API_STATUS)0xC0000007L)
#define SD_API_STATUS_NO_SUCH_DEVICE                   ((SD_API_STATUS)0xC0000008L)
#define SD_API_STATUS_INVALID_DEVICE_REQUEST           ((SD_API_STATUS)0xC0000009L)
#define SD_API_STATUS_NO_MEMORY                        ((SD_API_STATUS)0xC000000AL)
#define SD_API_STATUS_BUS_DRIVER_NOT_READY             ((SD_API_STATUS)0xC000000BL)
#define SD_API_STATUS_DATA_ERROR                       ((SD_API_STATUS)0xC000000CL)
#define SD_API_STATUS_CRC_ERROR                        ((SD_API_STATUS)0xC000000DL)
#define SD_API_STATUS_INSUFFICIENT_RESOURCES           ((SD_API_STATUS)0xC000000EL)
#define SD_API_STATUS_DEVICE_NOT_CONNECTED             ((SD_API_STATUS)0xC0000010L)
#define SD_API_STATUS_DEVICE_REMOVED                   ((SD_API_STATUS)0xC0000011L)
#define SD_API_STATUS_DEVICE_NOT_RESPONDING            ((SD_API_STATUS)0xC0000012L)
#define SD_API_STATUS_CANCELED                         ((SD_API_STATUS)0xC0000013L)
#define SD_API_STATUS_RESPONSE_TIMEOUT                 ((SD_API_STATUS)0xC0000014L)
#define SD_API_STATUS_DATA_TIMEOUT                     ((SD_API_STATUS)0xC0000015L)
#define SD_API_STATUS_DEVICE_RESPONSE_ERROR            ((SD_API_STATUS)0xC0000016L)
#define SD_API_STATUS_DEVICE_UNSUPPORTED               ((SD_API_STATUS)0xC0000017L)
#define SD_API_STATUS_SHUT_DOWN                        ((SD_API_STATUS)0xC0000018L)
#define SD_API_STATUS_INSUFFICIENT_HOST_POWER          ((SD_API_STATUS)0xC0000019L)



typedef struct _csd_ {
	DWORD	c3;
	DWORD 	c2;
	DWORD	c1;
	DWORD 	c0;
}CSD, *pCSD;
/*
//SD card : CSD register bit field.
#define CSD_STRUCTURE			126		// 2bit
#define TAAC						112    	// 8bit
#define NSAC						104		// 8bit
#define TRANS_SPEED				96		// 8bit
#define CCC						84		// 12bit
#define READ_BL_LEN				80		// 4bit
#define READ_BL_PARTIAL			79		// 1bit
#define WRITE_BLK_MISALIGN		78		// 1bit
#define READ_BLK_MISALIGN		77		// 1bit
#define DSR_IMP					76		// 1bit
#define C_SIZE					62		// 12bit
#define VDD_R_CURR_MIN			59		// 3bit
#define VDD_R_CURR_MAX			56		// 3bit
#define VDD_W_CURR_MIN		53		// 3bit
#define VDD_W_CURR_MAX		50		// 3bit
#define C_SIZE_MULT				47		// 3bit
#define ERASE_BLK_EN			46		// 1bit
#define SECTOR_SIZE				39		// 7BIT
#define WP_GRP_SIZE				32		// 7BIT
#define WP_GRP_ENABLE			31		// 1BIT
#define R2W_FACTOR				26		// 3BIT
#define WRITE_BL_LEN			22		// 4bit
#define WRITE_BL_PARTIAL		21		// 1bit
#define FILE_FORMAT_GRP		15		// 1BIT
#define COPY						14		// 1BIT
#define PERM_WRITE_PROTECT		13		// 1bit
#define TMP_WRITE_PROTECT		12		// 1BIT
#define FILE_FORMAT				11		// 2bit
*/
#define BUS_1_BIT       0
#define BUS_4_BIT       1
#define CMD0 		0	// software reset
#define CMD1 		1	// not use for sd
#define CMD2             2	// check OSR
#define CMD3		3	// get RCA
#define CMD4		4
#define CMD7		7	// select / deselect
#define CMD8		8	// detect sd2.0
#define CMD9		9
#define CMD10		10	// get 
#define CMD12		12	// from sending data state to transfer state .
#define CMD15		15	// into Inactive .
#define CMD16		16	// set block len.
#define CMD17		17	// single block read
#define CMD18		18	// multi block read
#define ACMD6		6
#define ACMD41		41	// instead of cmd1, 
#define ACMD51		51	// ask SCR
#define CMD55		55	// before acmd


typedef struct _sd_card_ {
	struct _sd_card_  *next;
	DWORD	RCA;
	DWORD   index;
	DWORD   status;
	DWORD	sector_size; 		// 
	DWORD	capicity;			// how many Mega bytes.
	DWORD	max_speed;			//  kbits/s
	DWORD	file_format;
	BOOL	Is_SD20;
	BOOL	Is_SDHC;

}SD_CARD, *pSD_CARD;

#ifdef __cplusplus
}
#endif
#endif /*__SD_H___*/
